Package: SpiecEasi
Title: Sparse Inverse Covariance for Ecological Statistical Inference
Version: 2.1.1
Date: 2026-04-29
Authors@R: c(
    person("Zachary", "Kurtz", role = c("aut", "cre"), email="zdkurtz@gmail.com"),
    person("Christian", "Mueller", role = "aut"),
    person("Emily", "Miraldi",     role="aut"),
    person("Richard", "Bonneau",   role="aut"),
    person("Laura", "Tipton",      role="ctb"))
Description: Estimate networks from the precision matrix of
        compositional microbial abundance data.
License: GPL (>= 3)
URL: https://github.com/zdk123/SpiecEasi
BugReports: https://github.com/zdk123/SpiecEasi/issues
biocViews: Software, Microbiome, Metagenomics, GraphAndNetwork,
        NetworkInference
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
Depends: R (>= 4.5.0),
Imports: stats, methods, graphics, grDevices, huge (>= 1.3.2), pulsar
        (>= 0.3.11), MASS, VGAM, Matrix (>= 1.5), glmnet, phyloseq
Suggests: parallel, boot, igraph, batchtools, testthat, covr, knitr,
        BiocStyle, rmarkdown, RefManageR, sessioninfo, magick
LinkingTo: Rcpp, RcppArmadillo
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/pak/sysreqs: libglpk-dev libicu-dev libxml2-dev zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-29 15:38:42 UTC
RemoteUrl: https://github.com/bioc/SpiecEasi
RemoteRef: HEAD
RemoteSha: 3c747724ed7bcb4a4f7f0cbf781dcfd2fc5395ee
NeedsCompilation: yes
Packaged: 2026-05-19 10:21:55 UTC; root
Author: Zachary Kurtz [aut, cre],
  Christian Mueller [aut],
  Emily Miraldi [aut],
  Richard Bonneau [aut],
  Laura Tipton [ctb]
Maintainer: Zachary Kurtz <zdkurtz@gmail.com>
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-19 10:27:17 UTC; unix
