# Generated by roxygen2: do not edit by hand

S3method('$',Matrix)
S3method('[[',Matrix)
S3method(alr,data.frame)
S3method(alr,default)
S3method(alr,matrix)
S3method(as.data.frame,graph)
S3method(as.matrix,graph)
S3method(clr,data.frame)
S3method(clr,default)
S3method(clr,matrix)
S3method(getOptX,pulsar.refit)
S3method(spiec.easi,default)
S3method(spiec.easi,list)
S3method(spiec.easi,otu_table)
S3method(spiec.easi,phyloseq)
export(adj2igraph)
export(alr)
export(clr)
export(coat)
export(cor2cov)
export(cov2prec)
export(ebic)
export(edge.diss)
export(fitdistr)
export(getOptBeta)
export(getOptCov)
export(getOptInd)
export(getOptLambda)
export(getOptMerge)
export(getOptNet)
export(getOptiCov)
export(getRefit)
export(getStability)
export(get_comm_params)
export(graph2prec)
export(make_graph)
export(multi.spiec.easi)
export(neff)
export(neighborhood.net)
export(norm_pseudo)
export(norm_rdiric)
export(norm_to_total)
export(prec2cov)
export(pval.sparccboot)
export(qqdplot_comm)
export(rmvnegbin)
export(rmvnorm)
export(rmvpois)
export(rmvzinegbin)
export(rmvzipois)
export(robustPCA)
export(rzipois)
export(shannon)
export(sparcc)
export(sparccboot)
export(sparseLowRankiCov)
export(sparseiCov)
export(spiec.easi)
export(stars.pr)
export(stars.roc)
export(symBeta)
export(synth_comm_from_counts)
export(tril)
export(triu)
export(triu2diag)
importFrom(MASS,ginv)
importFrom(Matrix,t)
importFrom(VGAM,dzinegbin)
importFrom(VGAM,dzipois)
importFrom(VGAM,qzinegbin)
importFrom(VGAM,qzipois)
importFrom(VGAM,rdiric)
importFrom(glmnet,glmnet)
importFrom(grDevices,dev.off)
importFrom(grDevices,png)
importFrom(graphics,abline)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(huge,huge)
importFrom(huge,huge.npn)
importFrom(methods,as)
importFrom(pulsar,batch.pulsar)
importFrom(pulsar,getLamPath)
importFrom(pulsar,getMaxCov)
importFrom(pulsar,pulsar)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,cov2cor)
importFrom(stats,dnbinom)
importFrom(stats,dpois)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,na.exclude)
importFrom(stats,na.omit)
importFrom(stats,optim)
importFrom(stats,pnorm)
importFrom(stats,ppoints)
importFrom(stats,qlnorm)
importFrom(stats,qnbinom)
importFrom(stats,qpois)
importFrom(stats,rmultinom)
importFrom(stats,rnorm)
importFrom(stats,rpois)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,var)
useDynLib(SpiecEasi)
